;;;; An LAML Article page style which emit LaTeX instead of HTML.
;;;; This style is for authoring a relatively simple scientific paper in LAML.
;;;; The article style is modelled after LaTeX, but it is still incomplete and
;;;; ad hoc. I include stuff when I need it. Simple formatting can be done via HTML tags mirrored in LAML.

; The LAML library and programs are written by Kurt Normark, Aalborg University, Denmark.
; Copyright (C) 1999  Kurt Normark, normark@cs.auc.dk.
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

; COPYRIGHT (c)  Kurt Nrmark, Department of Computer Science, Aalborg University, Denmark.
; normark@cs.auc.dk,  http://www.cs.auc.dk/~normark/

; ---------------------------------------------------------------------------------------------------
; LOADING

; Assumes that laml.scm is preloaded by the context of this style.
(lib-load "latex.scm")
(lib-load "file-read.scm")

; ---------------------------------------------------------------------------------------------------
; VARIABLES WHICH HOLD ARTICLE CONSTITUENTS

(define laml-source-file-name-without-extension (source-filename-without-extension))
(define article-title "Article")
(define article-author "Author")
(define article-affiliation '())
(define article-abstract "")
(define home-url #f)

(define article-name laml-source-file-name-without-extension)

(define article-section-list '()) ; the list in which we collect article section descriptions
(define bibliography-item-list '())

(define aux-info '())

; -----------------------------------------------------------------------------
; Setter functions:

(define (set-article-name! name)
  (set! article-name name))

(define (set-article-author! author-and-affiliation)
  (if (and (list? author-and-affiliation) (>= (length author-and-affiliation) 1))
      (set! article-author (car author-and-affiliation))
      (set! article-author "Author")))

(define (set-article-affiliation! author-and-affiliation)
  (if (and (list? author-and-affiliation) (>= (length author-and-affiliation) 1))
      (set! article-affiliation (cdr author-and-affiliation))
      (set! article-affiliation (list "Affiliation"))))

(define (set-article-abstract! abstract)
  (set! article-abstract abstract))

(define (set-article-title! title)
  (set! article-title title))

(define (set-home-url! url)
  (set! home-url url))


; ---------------------------------------------------------------------------------------------------
;;; Top level function. 
;;; In this section the important top level functions of the article style are presented. 

;; Define the article title
(define (article-title title)
  (set-article-title! title))

;; Define the article author and affiliation. 
;; The first of the parameters is usually the author name. The remaining parameters
;; represent affiliation information.
(define (article-author-and-affiliation . author-and-affiliation)
  (set-article-author! author-and-affiliation)
  (set-article-affiliation! author-and-affiliation))

;; Define the article abstract
(define (article-abstract abstract)
  (set-article-abstract! abstract))

;; Define a section in this article. 
;; The first parameter, id, is a symbol which identifies the section.
;; The seconde parameter, title, is a title string. The last parameter
;; is the body of a section, typically and article-paragraph.
(define (article-section id title section-body)
  (set! article-section-list 
    (cons 
     (make-sectional-entry 'section id (list 0) title section-body)
     article-section-list)))

;; Define a subsection in this article. 
;; The first parameter, id, is a symbol which identifies the section.
;; The seconde parameter, title, is a title string. The last parameter
;; is the body of a section, typically and article-paragraph.
(define (article-subsection id title section-body)
 (set! article-section-list 
    (cons 
     (make-sectional-entry 'subsection id (list 0 0) title section-body)
     article-section-list)))

;; Define a number of paragraphs. 
;; Paragraphs are separated by a p tag.
(define (article-paragraphs . paragraph-list)
  (let* ((lgt (length paragraph-list))
         (separator (string-append CR CR))
         (separator-list (make-list (- lgt 1) separator)))
    (string-merge paragraph-list separator-list)))

;; Define an article figure in terms of an id, a relative file path, and a caption. 
;; The latex version assumes an eps file.
;; The first parameter, id, is a symbol which identifies the figure.
;; The second parameter, relative-file-path, is the relative file path to the eps file (without extension).
;; The third parameter is the caption of the figure (a string).
;; The latex-info parameter is the column-span (single or double) and width.
(define (article-figure id relative-file-path caption . latex-info)
 (let ((column-span (if (null? latex-info) 'double (car latex-info)))
       (width (if (null? latex-info) "100mm" (cadr latex-info))))
  
   (if (null? latex-info)
       (display-warning (string-append "article-figure: " (as-string id) ". " "No latex-info (column-span and width) present. Using defaults.")))

   (string-append CR
    (latex-environment-1
      (if (eq? column-span 'single) 'figure 'figure*) "tb"
      (string-append
        (latex-environment 'center
         (string-append
           "\\leavevmode" CR
           "\\epsfxsize=" width CR
           "\\epsfclipon" CR
           (latex-command 'epsfbox (string-append relative-file-path ".eps")) CR
         )
        )
        CR
        (latex-command 'lcaption caption) CR
        (latex-command 'label id) CR))
    CR)))

      


;; Define a number of name issues, which are rendered as titled items in a list.
;; The parameters are supposed to be issue forms.
(define (article-named-issue-list . issue-list)
 (apply article-items
  (map format-issue issue-list)))

(define (format-issue issue)
  (let ((issue-header (second issue))
        (issue-body (third issue)))
    (latex-command 'item (string-append (b issue-header) ": " issue-body))))


;; Define an issue of an article-named-issue-list.
;; An issue is defined in terms of a title and a body.
(define (issue issue-title issue-body)
  (list 'issue issue-title issue-body))

;; Define a number of items.
;; The parameters are supposed to be item forms.
(define (article-items . items)
 (latex-environment 'itemize
    (list-to-string items CR)))

;; Define an item in an article-items form.
(define (item i)
 (string-append (latex-command 'item) " " i))
  

;; The clause which starts the article proper.
;; This clause must be ended by end-article.
;; Put article-title, article-author-and-affiliation, article-abstract, the sections, and the article-bibliography
;; between begin-article and end-article.
(define (begin-article)
  #t)


;; The clause which ends the article.
(define (end-article)  ; LaTeX specific
 (let* ((dir (startup-directory))
        (file article-name)
        (extension "tex")
        (file-path (string-append dir file "." extension)))
  (write-text-file
    (page article-title
          (the-article)
    )
    file-path)

  ; remove Control M characters:
  (load (string-append laml-dir "utilities/" "char-tool.scm"))
  (dos-to-unix file-path)

  (if (eq? laml-platform 'unix)
    (begin
      ; run latex, bibtex and xdvi:
      (run-latex dir file)
      (run-bibtex dir file)
      (run-latex dir file)
      (run-latex dir file)
      (run-xdvi dir file)))
))



;; This form returns a link (anchor tag) to a section or subsection of the article.
;; The reference is resolved via the aux file.
(define (section-ref id)
 (latex-command 'ref id))

;; This form returns a link to a figure of the article.
(define (figure-ref id)
 (latex-command 'ref id))

;; This form returns a list of links to citations of this article.
;; key-list is a list of strings, which identify bibliograph items.
(define (cite . key-list)
  (latex-command 'cite (list-to-string (map as-string key-list) ",")))

; Assume for now that all bib items are cited

;; Define the bibliograph items of the article. 
;; It is assumed that the constituents are bib-item forms.
(define (article-bibliography . bib-items)
  "")
  
;; The individual bibliograph items of the article-bibliography.
;; A bib-item is defined in terms of a key and a string, which represents the formatted bib information.
(define (bib-item key formatted-bib)
  "")

; ---------------------------------------------------------------------------------------------------
; Helping functions to top level forms


(define (make-sectional-entry unit id number-list title body)
  (list unit id number-list title body))

(define get-unit-of-entry (make-selector-function 1))
(define get-id-of-entry (make-selector-function 2))
(define get-number-list-of-entry (make-selector-function 3))
(define get-title-of-entry (make-selector-function 4))
(define get-body-of-entry (make-selector-function 5))



(define (format-section-unit sectional-entry)
  (let (
        (unit        (get-unit-of-entry sectional-entry))
        (id          (get-id-of-entry sectional-entry))
        (number-list (get-number-list-of-entry sectional-entry))
        (title       (get-title-of-entry sectional-entry))
        (body        (get-body-of-entry sectional-entry))
       )
    (string-append
      (latex-command unit title) CR
      (latex-command 'label id) CR
      body
    )))

(define (make-cite-entry key number)
  (list key number))

; ---------------------------------------------------------------------------------------------------------------
; Latex specific stuff:

(define (page title content)
  (string-append

    auto-generation-warning

    (read-text-file 
      (string-append (startup-directory) "templates/" "article-prefix"))  ; including anyting until \begin{document}

    CR

    content CR 

    (read-text-file 
      (string-append (startup-directory) "templates/" "article-suffix"))  ; bibliographstyle, bibliograph, and \end{document}

    CR
  )
)

(define (the-article)
  (string-append

    (latex-command 'title article-title) CR

    (latex-command 'author 
      (list-to-string 
        (cons (latexify-danish-letters article-author) article-affiliation) 
        latex-force-linekbreak))

    CR

    (latex-command 'maketitle) CR

    (latex-environment 'abstract article-abstract) CR

    (list-to-string
      (map format-section-unit (reverse article-section-list))
      (string-append (as-string #\newline) (as-string #\newline))) CR

  )
)

; ---------------------------------------------------------------------------------------------------------------
;;; Target specific insertions. Do only use these in special situations.

; Insert insertion if we generate latex
(define (latex-only insertion)
  insertion)

; Insert insertion if we generate html.
(define (html-only insertion)
  "")


(laml-welcome)






